@@ -259,6 +259,11 @@ impl<N: Num> Line<N> {
         FromRounded::from_rounded(angle)
     }
 
+    pub fn direction(self) -> Size<f32> {
+        let angle = self.to_f32().angle();
+        Size(angle.cos(), angle.sin())
+    }
+
     pub fn hypot(self) -> N {
         self.start().hypot_to(self.end())
     }
@@ -331,7 +336,10 @@ impl<N: Num + Signed> Line<N> {
     }
 }
 
-impl<N: Num + ToSignedClamped> Line<N> {
+impl<N: Num + ToSignedClamped> Line<N>
+where
+    <N as ToSignedClamped>::Output: Signed,
+{
     pub fn to_signed_clamped(self) -> Line<<N as ToSignedClamped>::Output> {
         Line(
             self.start().to_signed_clamped(),
@@ -360,6 +368,10 @@ impl<N: Num + ToSignedClamped> Line<N> {
         let diff = self.diff();
         diff.width() / diff.height()
     }
+
+    pub fn step_direction(self) -> Point<<N as ToSignedClamped>::Output> {
+        self.diff().to_point().sign()
+    }
 }
 
 impl Line<f32> {
