@@ -43,6 +43,22 @@ impl<N: Num> Rect<N> {
         Self(centre - size.half(), size)
     }
 
+    pub fn new_from_top_left(position: Point<N>, size: Size<N>) -> Self {
+        Self(position - Point(N::zero(), size.height()), size)
+    }
+
+    pub fn new_from_top_right(position: Point<N>, size: Size<N>) -> Self {
+        Self(position - size.to_point(), size)
+    }
+
+    pub fn new_from_bottom_right(position: Point<N>, size: Size<N>) -> Self {
+        Self(position - Point(size.width(), N::zero()), size)
+    }
+
+    pub fn new_from_bottom_left(position: Point<N>, size: Size<N>) -> Self {
+        Self(position, size)
+    }
+
     pub fn move_xy(&mut self, xy: Point<N>) {
         self.0 += xy;
     }
@@ -234,12 +250,59 @@ impl<N: Num> Rect<N> {
         Line(self.centre(), other.centre())
     }
 
+    pub fn left_edge(self) -> Line<N> {
+        Line(self.bottom_left(), self.top_left())
+    }
+
+    pub fn right_edge(self) -> Line<N> {
+        Line(self.bottom_right(), self.top_right())
+    }
+
+    pub fn top_edge(self) -> Line<N> {
+        Line(self.top_left(), self.top_right())
+    }
+
+    pub fn bottom_edge(self) -> Line<N> {
+        Line(self.bottom_left(), self.bottom_right())
+    }
+
     pub fn interpolate_to(self, other: Rect<N>, n: N) -> Rect<N> {
         let new_centre = self.centre().interpolate_to(other.centre(), n);
         let new_size = self.size().interpolate_to(other.size(), n);
 
         Rect::new_from_centre(new_centre, new_size)
     }
+
+    pub fn round_to_max_size(self) -> Rect<N> {
+        let self_f32 = self.to_f32();
+        let Point(bottom_left_x, bottom_left_y) = self_f32.bottom_left();
+        let Point(top_right_x, top_right_y) = self_f32.top_right();
+
+        let new_bottom_left_x;
+        let new_bottom_left_y;
+        let new_top_right_x;
+        let new_top_right_y;
+
+        if bottom_left_x < top_right_x {
+            new_bottom_left_x = bottom_left_x.floor();
+            new_top_right_x = top_right_x.ceil();
+        } else {
+            new_bottom_left_x = bottom_left_x.floor();
+            new_top_right_x = top_right_x.ceil();
+        }
+
+        if bottom_left_y < top_right_y {
+            new_bottom_left_y = bottom_left_y.floor();
+            new_top_right_y = top_right_y.ceil();
+        } else {
+            new_bottom_left_y = bottom_left_y.floor();
+            new_top_right_y = top_right_y.ceil();
+        }
+
+        let new_rect_f32 = Point(new_bottom_left_x, new_bottom_left_y)
+            .rect_to(Point(new_top_right_x, new_top_right_y));
+        new_rect_f32.from_f32()
+    }
 }
 
 impl Rect<f32> {
@@ -453,6 +516,58 @@ mod new_from_centre {
     }
 }
 
+#[cfg(test)]
+mod new_from_top_left {
+    use super::*;
+
+    #[test]
+    fn it_should_place_rect_correctly() {
+        let rect: Rect<i32> = Rect::new_from_top_left(Point(10, 100), Size(10, 20));
+        let expected: Rect<i32> = Rect(Point(10, 80), Size(10, 20));
+
+        assert_eq!(rect, expected);
+    }
+}
+
+#[cfg(test)]
+mod new_from_top_right {
+    use super::*;
+
+    #[test]
+    fn it_should_place_rect_correctly() {
+        let rect: Rect<i32> = Rect::new_from_top_right(Point(10, 100), Size(10, 20));
+        let expected: Rect<i32> = Rect(Point(0, 80), Size(10, 20));
+
+        assert_eq!(rect, expected);
+    }
+}
+
+#[cfg(test)]
+mod new_from_bottom_left {
+    use super::*;
+
+    #[test]
+    fn it_should_place_rect_correctly() {
+        let rect: Rect<i32> = Rect::new_from_bottom_left(Point(10, 100), Size(10, 20));
+        let expected: Rect<i32> = Rect(Point(10, 100), Size(10, 20));
+
+        assert_eq!(rect, expected);
+    }
+}
+
+#[cfg(test)]
+mod new_from_bottom_right {
+    use super::*;
+
+    #[test]
+    fn it_should_place_rect_correctly() {
+        let rect: Rect<i32> = Rect::new_from_bottom_right(Point(10, 100), Size(10, 20));
+        let expected: Rect<i32> = Rect(Point(0, 100), Size(10, 20));
+
+        assert_eq!(rect, expected);
+    }
+}
+
 #[cfg(test)]
 mod bottom_left {
     use super::*;
@@ -639,3 +754,83 @@ mod interpolate_to {
         );
     }
 }
+
+#[cfg(test)]
+mod round_to_max_size {
+    use super::*;
+    use crate::geom::testing_utils::assert_approx_rect_eq;
+    use ::testcat::*;
+
+    it!(
+        "should enlarge positive rectangles to make size larger",
+        test_positive_rect
+    );
+    it!(
+        "should enlarge positive rectangles with an inverse size",
+        test_positive_rect_with_negative_size
+    );
+    it!(
+        "should enlarge negative rectangles to make size larger",
+        test_negative_rect
+    );
+    it!(
+        "should enlarge negative rectangles with an inverse size",
+        test_negative_rect_with_negative_size
+    );
+    it!(
+        "should enlarge rectangles that cross zero to make size larger",
+        test_cross_zero_rect
+    );
+    it!(
+        "should enlarge rectangles that cross zero with an inverse size",
+        test_cross_zero_rect_with_negative_size
+    );
+
+    fn test_positive_rect() {
+        let rect: Rect<f32> = Point(10.2, 10.8).rect_to(Point(19.2, 19.8));
+        let rounded = rect.round_to_max_size();
+        let expected: Rect<f32> = Point(10.0, 10.0).rect_to(Point(20.0, 20.0));
+
+        assert_approx_rect_eq(expected, rounded);
+    }
+
+    fn test_positive_rect_with_negative_size() {
+        let rect: Rect<f32> = Point(19.2, 19.8).rect_to(Point(10.2, 10.8));
+        let rounded = rect.round_to_max_size();
+        let expected: Rect<f32> = Point(20.0, 20.0).rect_to(Point(10.0, 10.0));
+
+        assert_approx_rect_eq(expected, rounded);
+    }
+
+    fn test_negative_rect() {
+        let rect: Rect<f32> = Point(-10.2, -10.8).rect_to(Point(-19.2, -19.8));
+        let rounded = rect.round_to_max_size();
+        let expected: Rect<f32> = Point(-10.0, -10.0).rect_to(Point(-20.0, -20.0));
+
+        assert_approx_rect_eq(expected, rounded);
+    }
+
+    fn test_negative_rect_with_negative_size() {
+        let rect: Rect<f32> = Point(-19.2, -19.8).rect_to(Point(-10.2, -10.8));
+        let rounded = rect.round_to_max_size();
+        let expected: Rect<f32> = Point(-20.0, -20.0).rect_to(Point(-10.0, -10.0));
+
+        assert_approx_rect_eq(expected, rounded);
+    }
+
+    fn test_cross_zero_rect() {
+        let rect: Rect<f32> = Point(-10.2, -10.8).rect_to(Point(19.2, 19.8));
+        let rounded = rect.round_to_max_size();
+        let expected: Rect<f32> = Point(-11.0, -11.0).rect_to(Point(20.0, 20.0));
+
+        assert_approx_rect_eq(expected, rounded);
+    }
+
+    fn test_cross_zero_rect_with_negative_size() {
+        let rect: Rect<f32> = Point(19.2, 19.8).rect_to(Point(-10.2, -10.8));
+        let rounded = rect.round_to_max_size();
+        let expected: Rect<f32> = Point(20.0, 20.0).rect_to(Point(-11.0, -11.0));
+
+        assert_approx_rect_eq(expected, rounded);
+    }
+}
